#!/usr/bin/perl
# Copyright 2006 Fonality, Inc. ALL RIGHTS RESERVED

use Digest::MD5 qw/md5_hex/;
use Sys::Hostname;
use LWP::Simple;
use strict;

# if this is a dell system we should NOT run
if (-e '/etc/dell')
{
	exit(1);
}

my %devices = map { $_ => 1 } qw/
	0004F2
	00E075
	00085D
/;

$|++;
my $debug = 1;
my(%macs, $user, $pass);
my($scan_cmd) = '/usr/bin/nmap';

my ($node) = hostname;
$node =~ s/(?:trixbox|pbxtra)//;

open(VPN, "</etc/sysconfig/vpn");
my $stype = 'pbxtra';
while (<VPN>)
{
    if (/^GW.*trixbox/)
    {
        $stype = 'trixbox';
        last;                                                                                         
    }                                                                                                 
}                                                                                                     
close(VPN);

open CONF, "</etc/asterisk/fonality/globals.conf" || die "Can't open globals.conf for reading: $!";
while (<CONF>)
{
    if (/IAXINFO\d?=(s\d{4,6}):([^;\s]+)/)
    {
        ($user, $pass) = ($1, $2);
        $user =~ s/[xi]$//;

        last if $user =~ /^s\d+$/;
    }
}
close(CONF);

# Check for programs we depend on
unless(-x $scan_cmd) {
	system("/usr/bin/wget http://restore.fonality.com/nmap-3.81-1.i386.rpm");
	system("rpm -i nmap-3.81-1.i386.rpm");
	unlink('nmap-3.81-1.i386.rpm');
}

# scan for existing devices
open(SIP, "</etc/asterisk/sip.conf") || die "Can't read sip.conf: $!";
while (<SIP>)
{
	if (/^\[(.*?)\]/)
	{
		$macs{$1} = 1;
	}
}
close(SIP);

my(%scanned_macs,$ip);

foreach my $eth (qw/eth0 eth1/)
{
my $ifconfig = `/sbin/ifconfig $eth 2>>/dev/null`;
my ($net) = ($ifconfig =~ /inet addr:(\S+\.)\d+\s/);
next unless $net;
dprint("network = $net\n");


dprint("Scanning for phones...");
my($cmd) = "$scan_cmd -sP ${net}1-255";
my(@lines) = `$cmd`;
dprint("done\n\n");

foreach my $l (@lines) {
	if($l =~ /^Host (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) {
		$ip = $1;
	}
	elsif ($l =~ /^MAC Address:\s+(\S+)/)
	{
		my $mac = $1;
		$mac =~ s/://g;
		if ($devices{substr($mac, 0, 6)})
		{
			$scanned_macs{$mac} = $ip;
		}
	}
}
}

my($count) = 0;
foreach my $key (sort(keys(%scanned_macs))) {
	next if(defined($macs{$key}));
	dprint("New device: $key = $scanned_macs{$key}\n");
	add_device($key);
	$count++;
}

if($count == 1) {
	dprint("\nFound $count new phone\n");
}
else {
	dprint("\nFound $count new phones\n");
}

sub dprint
{
	print STDERR $_[0] if $debug;
}

sub add_device
{
    my $desc;
    if ($devices{substr($_[0], 0, 6)})
    {
	# if it's a polycom phone that's being auto-provisioned
	# we want to find a description for it
	if(substr($_[0],0,6) =~ /0004F2|00E075/i)
	{
	    $desc = search_bootlog($_[0]);
	    if ($desc)
	    {
		dprint("Phone model found: $desc\n");
	    }
	    else
	    {
		exit(0);
	    }
	}
    }

	my $resp = get("http://cp." . ($stype eq 'trixbox' ? 'trixbox' : 'fonality') . ".com/add_devices.cgi?server_id=$node&device=$_[0]&description=$desc&hash=" . md5_hex($_[0] . $pass));
	$resp =~ s/\n/ /g;
	print "$resp\n";
}

sub search_bootlog
{
    my $mac = lc(shift);
    my $file = "/tftpboot/$mac-boot.log";
    my $desc;

    # if the bootlog doesn't exist, tell the user to reboot their phone
    unless( -e $file)
    {
	print "Cannot find $file. Please reboot your phone (MAC: ".uc($mac).")\n";
	print "Re-run this option after the phone has booted and it has not been automatically detected.\n";
	unless (-e '/etc/dell')
	{
	    print "If the trouble persists please add this device manually in the Extension/Phones page of the Web Admin User Interface.\n";
	}
	return (0);
    }

    # search through the bootlog to find the phone model
    open(BOOTLOG, $file) || dprint("Can't read $file: $!");
    while(<BOOTLOG>)
    {
        # matches:
        # Platform: Model=SoundPoint IP 650, Assembly=2345-12600-001 Rev=B
        if(/Platform\: Model=\D+(\d+)\D/)
        {
            $desc = $1;
            last;
        }
    }
    close BOOTLOG;

    # description becomes: Auto-detected NNN

    return $desc;
}
