#!/usr/bin/perl
#
# sync_greet_files.pl - task to keep voicemail greetings sync'ed on linked servers
# -jlawrence@fonality.com
use strict;

# nice ourselves to 5
system("/usr/bin/renice 5 $$ >>/dev/null 2>>/dev/null");

# Check if sync_linked_greet is already running.
check_running();

# Get correct numerical UID/GID for user/group 'nobody'
my ($nobody_uid, $nobody_gid) = (getpwnam("nobody"))[ 2, 3 ];

# Get our server-id so we can make sure we don't sync to it
open(GLOBALS, "</etc/asterisk/fonality/globals.conf");
my @lines = <GLOBALS>;
close(GLOBALS);

my $server_id;

foreach my $line (@lines)
{
	if ($line =~ /ACCOUNT=(\d{4,})/)
	{
		$server_id = $1;
	}
}

# Let's check link.conf to see if we have any linked servers before we do anything else
open(LINKS, "</etc/asterisk/fonality/link.conf");
my @links = <LINKS>;
close(LINKS);

# did we find it?
my $found = undef;

# loop through the file data and check for a link
foreach my $link (@links)
{
	if ($link =~ /exten/)
	{
		$found = 1;
	}
}

# no links, bail out
unless ($found)
{
	exit(1);
}

# Get a list of linked servers
my @raw_server_list = split(/\n/, `/usr/sbin/asterisk -rx 'iax2 show peers'`);

# Store the servers we find here
my %links;

# put the servers we are connected to into the %links hash
foreach my $server_line (@raw_server_list)
{
	if ($server_line =~ /c(\d{4,})(?:i|x).*OK/)
	{
		unless ($server_id == $1)
		{
			$links{$1} = 1;
		}
	}
} 

if (scalar %links eq "0")
{

	# No linked servers
	exit;
}

# Write up-to-the-hour dump file of all greet* files in the voicemail folder
# (more resolution in time may cause sync'er to constantly re-sync from minute date changes)
my $greet_text;

opendir(DIR, "/var/spool/asterisk/voicemail/default");
my @dirs = readdir(DIR);
closedir(DIR);

foreach my $dir (@dirs)
{
	my $filename = "/var/spool/asterisk/voicemail/default/$dir/greet.wav";
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($filename);
	
	if ($mtime)
	{
		$greet_text .= "$mtime $filename\n";	
	}
	
	my $filename = "/var/spool/asterisk/voicemail/default/$dir/greet.WAV";
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($filename);
	
	if ($mtime)
	{
		$greet_text .= "$mtime $filename\n";	
	}
	
	my $filename = "/var/spool/asterisk/voicemail/default/$dir/greet.gsm";
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($filename);
	
	if ($mtime)
	{
		$greet_text .= "$mtime $filename\n";	
	}
}

my $greet_file = "";
open GREETFILE, "/var/lib/asterisk/greet.list";
while (<GREETFILE>) { $greet_file .= $_; }
close GREETFILE;

if ($greet_file eq $greet_text || $greet_text eq "")
{

	# No changes, no need to proceed
	exit;
}

if ($greet_file eq "")
{

	# If file doesn't exist, create it and exit.
	open GREETFILE, ">/var/lib/asterisk/greet.list"
	  || die "Cannot open /var/lib/asterisk/greet.list for writing: $@";
	print GREETFILE $greet_text;
	close GREETFILE;
	exit;
}
else
{

	# Get extensions local to this PBXtra box - taken from /var/lib/asterisk/agi-bin/getexten.agi
	my %extensions;
	open(CONF, "</etc/asterisk/fonality/external.conf")
	  || die "Can't read external.conf: $!";
	while (<CONF>)
	{
		if (/^exten => (\d+),1,Macro\([^,]+,[^,]+,[^,]+,([^),]+)/)
		{
			$extensions{$1} = 1;
		}
	}
	close CONF;

	open(CONF, "</etc/asterisk/fonality/internal.conf")
	  || die "Can't read internal.conf: $!";
	while (<CONF>)
	{
		if (/^exten => (\d+),1,Macro\([^,]+,[^,]+,[^,]+,([^),]+)/)
		{
			$extensions{$1} = 1;
		}
	}
	close CONF;

	# Otherwise, check text buffer vs. file contents
	my @greet_file = split("\n", $greet_file);
	my @greet_text = split("\n", $greet_text);

	foreach my $line_in_file (@greet_file)
	{
		my $file_found    = 0;
		my @line_in_file  = split(" ", $line_in_file);
		my $file_date     = $line_in_file[0];
		my $file_filename = @line_in_file[1];

		foreach my $line_in_text (@greet_text)
		{
			my @line_in_text  = split(" ", $line_in_text);
			my $text_date     = $line_in_text[0];
			my $text_filename = @line_in_text[1];

			if ($text_filename eq $file_filename)
			{
				$file_found = 1;
				if ($text_date != $file_date)
				{

					# File date changed, sync to linked server(s)
					print "$file_filename date changed...\n";
					my @file_ext = split("\\.", $file_filename);
					my @vm_ext   = split("/",   $file_filename);

					if (defined $extensions{ $vm_ext[6] })
					{
						foreach my $link (keys %links)
						{
							my $greet_callfile =
							  "greet" . int(rand(9999)) . ".call";
							open CALLFILE, ">/tmp/$greet_callfile";
							print CALLFILE "Channel: Local/VMR205-" 
							  . $link . "-"
							  . $vm_ext[6]
							  . "-greet-"
							  . $file_ext[1]
							  . '@internal' . "\n";
							print CALLFILE "Context: internal\n";
							print CALLFILE "SetVar: VMEXTEN="
							  . $vm_ext[6] . "\n";
							print CALLFILE "SetVar: VMFILE=greet\n";
							print CALLFILE "SetVar: VMFORMAT="
							  . $file_ext[1] . "\n";
							print CALLFILE "Extension: VMR206\n";
							print CALLFILE "Priority: 1\n";
							close CALLFILE;

							chown $nobody_uid, $nobody_gid,
							  "/tmp/$greet_callfile"
							  || die "Error in chown... $@";
							rename "/tmp/$greet_callfile",
							  "/var/spool/asterisk/outgoing/$greet_callfile";

							# Take nap until call file is processed
							while (
								-e "/var/spool/asterisk/outgoing/$greet_callfile"
							  )
							{
								sleep(1);
							}
						}
					}
				}
			}

			# File is new, sync to linked server(s)
			if ($file_found == 0)
			{
				print "New greeting found: $text_filename\n";
				my @file_ext = split("\\.", $text_filename);
				my @vm_ext   = split("/",   $text_filename);

				if (defined $extensions{ $vm_ext[6] })
				{
					foreach my $link (keys %links)
					{
						my $greet_callfile =
						  "greet" . int(rand(9999)) . ".call";
						open CALLFILE, ">/tmp/$greet_callfile";
						print CALLFILE "Channel: Local/VMR205-" 
						  . $link . "-"
						  . $vm_ext[6]
						  . "-greet-"
						  . $file_ext[1]
						  . '@internal' . "\n";
						print CALLFILE "Context: internal\n";
						print CALLFILE "SetVar: VMEXTEN=" . $vm_ext[6] . "\n";
						print CALLFILE "SetVar: VMFILE=greet\n";
						print CALLFILE "SetVar: VMFORMAT="
						  . $file_ext[1] . "\n";
						print CALLFILE "Extension: VMR206\n";
						print CALLFILE "Priority: 1\n";
						close CALLFILE;

						chown $nobody_uid, $nobody_gid,
						  "/tmp/$greet_callfile" || die "Error in chown... $@";
						rename "/tmp/$greet_callfile",
						  "/var/spool/asterisk/outgoing/$greet_callfile";

						# Take nap until call file is processed
						while (
							-e "/var/spool/asterisk/outgoing/$greet_callfile")
						{
							sleep(1);
						}
					}
				}
			}
		}
	}

	# After it's all said and done, replace file with up-to-date buffer
	open GREETFILE, ">/var/lib/asterisk/greet.list";
	print GREETFILE $greet_text;
	close GREETFILE;
}

# See if we're running by proc name.
sub check_running
{
	my $proc = $0;
	$proc =~ s|^.*?([^/]+)$|$1|;
	my %ps = psgrep($proc);
	delete $ps{$$} if exists $ps{$$};
	my @procs = keys(%ps);

	if (scalar(@procs) > 0)
	{
		die "$proc already running as PID $procs[0]!";
	}
}

# Safe (non `ps`) ps + grep combo
sub psgrep
{
	my (@grep) = @_;
	my $g = $grep[0];

	my %return;

	opendir(PROC,"/proc") || return undef;
	foreach my $pid (grep { /^\d+$/ } readdir(PROC))
	{
		open (CMD, "</proc/$pid/cmdline") || next;
		my $cmdline = <CMD>;
		close (CMD);

		# no name
		if (!$cmdline)
		{
			open(STAT, "</proc/$pid/stat") || next;
			($cmdline) = (split(/ /,<STAT>))[1];
			close(STAT);

			$cmdline =~ s/[()]//g;
		}

		# if everything matches
		if (@grep == 1)
		{
			if ($cmdline =~ /^(?:(?:[^\0]*\/)?(?:sh|perl)\0)?(?:[^\0]*\/)?[^\/\0]*$g[^\/\0]*\0|^$g$/)
			{
				$cmdline =~ s/\0$//;
				$cmdline =~ s/\0/ /g;
				$return{$pid} = $cmdline;
			}
		}
		elsif (@grep != 1)
		{
			unless (grep { $cmdline !~ /$_/ } @grep)
			{
				$cmdline =~ s/\0$//;
				$cmdline =~ s/\0/ /g;
				$return{$pid} = $cmdline;
			}
		}
	}
	closedir(PROC);

	return %return;
}
